package utils;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.TimeUnit;

import model.*;

public class ClientThread extends Thread {

	private enum State
	{
		ActionRequested,
		ActionReady,
		DisconnectRequested,
		InitRequested
	}

	private static final long TIME_LIMIT_PER_TICK = 2;
	
	private Client client;
	private Rules rules;
	private BlockingQueue<State> qin = new LinkedBlockingDeque<>(),
			qout = new LinkedBlockingDeque<>();
	
	volatile private Action userAction;
	volatile private Player curPlayer;
	volatile private World curWorld;
	volatile private boolean crashed;

	public ClientThread(Client client, Rules rules)
	{
		this.client = client;
		this.userAction = null;
		crashed = false;
	}
	
	@Override
	public void run()
	{
		try
		{
			while(true)
			{
				State nextState = qin.take();
				if(nextState == State.DisconnectRequested)
					break;
				else if(nextState == State.ActionRequested)
				{
					userAction = null;
					userAction = client.getAction(curPlayer, curWorld);
					qout.put(State.ActionReady);
				}
				else if(nextState == State.InitRequested)
				{
					client.init(rules);
				}
			}
		}
		catch(InterruptedException e)
		{
			e.printStackTrace();
		}
		finally
		{
			client.disconnect();
		}
	}

	public void requestUserAction(Player player, World world)
	{
		try
		{
			curPlayer = player;
			curWorld = world;
			qin.put(State.ActionRequested);
		}
		catch(InterruptedException e)
		{
			e.printStackTrace();
		}
	}
	
	public Action getUserAction()
	{
		Action result = null;
		
		try
		{
			State threadState = qout.poll(TIME_LIMIT_PER_TICK, TimeUnit.SECONDS);
			if(threadState == State.ActionReady)
			{
				result = this.userAction;
				this.userAction = null;
			}
		}
		catch(InterruptedException e)
		{
			e.printStackTrace();
		}
		if(result == null)
		{
			this.crashed = true;
			result = new Action();
		}
		result.normalize(rules);
		return result;
	}
	
	public long getTimeConsumed()
	{
		return client.getTotalTime();
	}
	
	public void init()
	{
		try
		{
			qin.put(State.InitRequested);
		}
		catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	public void disconnect()
	{
		try
		{
			qin.put(State.DisconnectRequested);
		}
		catch(InterruptedException e)
		{
			e.printStackTrace();
		}
	}

	public void setCrashed(boolean crashed)
	{
		this.crashed = crashed;
	}
	
	public boolean hasCrashed()
	{
		return crashed;
	}
}
